#!/usr/bin/env node
"use strict";

var _slicedToArray = require("babel-runtime/helpers/sliced-to-array")["default"];

var _interopRequire = require("babel-runtime/helpers/interop-require")["default"];

var Os = _interopRequire(require("os"));

var Fs = _interopRequire(require("fs"));

var Path = _interopRequire(require("path"));

var debuglog = _interopRequire(require("debuglog"));

var minimist = _interopRequire(require("minimist"));

var Child = _interopRequire(require("child_process"));

var Utils = _interopRequire(require("../lib/utils"));

var StormSubmitter = _interopRequire(require("../lib/storm_submitter"));

var debug = debuglog("cyclone");
var argv = minimist(process.argv.slice(2), {
    alias: {
        topology: "t"
    },
    "default": {
        topology: process.env.npm_package_main || "."
    }
});

var topology = argv.topology;

var rel = topology;
var abs = Path.resolve(topology);
var root = Utils.findroot(abs);

if (rel === abs) {
    // An absolute path was provided, so derive a path
    // relative to the module root (being `package.json`)
    rel = Utils.relative(abs);
}

if (Fs.existsSync(abs)) {
    if (Fs.statSync(abs).isDirectory()) {
        // Exists, but is a directory (e.g. '.') so find
        // `main` from package.json and update paths
        var pkg = require(Path.join(root, "package.json"));
        rel = pkg.main;
        abs = Path.resolve(pkg.main);
    }
} else {
    throw new Error("Unable to locate topology " + topology);
}

debug("Topology (absolute):", abs);
debug("Topology (relative):", rel);
debug("Module root:", root);

var tmp = Path.join(Os.tmpdir(), "cyclone-" + Utils.randInt(0, 10000000));
var resources = Path.join(tmp, "resources");

debug("Creating tmp directory " + resources);
Utils.mkdirp(resources);

debug("Copying resources to " + resources);
Utils.cp(root, resources);

var storm = Child.spawn("storm", ["shell", "resources/", "node", Path.join("resources", rel)], { stdio: ["ignore"], cwd: tmp });

var _storm$stdio = _slicedToArray(storm.stdio, 3);

var _ = _storm$stdio[0];
var stdout = _storm$stdio[1];
var stderr = _storm$stdio[2];

stdout.pipe(process.stdout);
stderr.pipe(process.stderr);