"use strict";

var _slicedToArray = require("babel-runtime/helpers/sliced-to-array")["default"];

var _core = require("babel-runtime/core-js")["default"];

var _interopRequire = require("babel-runtime/helpers/interop-require")["default"];

var debuglog = _interopRequire(require("debuglog"));

var Bolt = _interopRequire(require("./lib/bolt"));

var Spout = _interopRequire(require("./lib/spout"));

var Dispatcher = _interopRequire(require("./lib/dispatcher"));

var LocalCluster = _interopRequire(require("./lib/local_cluster"));

var StormSubmitter = _interopRequire(require("./lib/storm_submitter"));

var TopologyBuilder = _interopRequire(require("./lib/topology_builder"));

var debug = debuglog("cyclone");
var DEFAULT_OPTIONS = {
    name: "topology",
    config: {}
};

module.exports = {

    Bolt: Bolt,

    Spout: Spout,

    Dispatcher: Dispatcher,

    TopologyBuilder: TopologyBuilder,

    StormSubmitter: StormSubmitter,

    LocalCluster: LocalCluster,

    run: function run(builder) {
        var options = arguments[1] === undefined ? { nimbus: {}, config: {} } : arguments[1];

        options = _core.Object.assign(DEFAULT_OPTIONS, options);

        var argv = process.argv.slice(2);
        if (argv.length) {

            var topology = builder.createTopology();

            if (argv[0] === "--local") {
                (function () {

                    // OPTION 1: Called locally to start a LocalCluster
                    var cluster = new LocalCluster();
                    cluster.submitTopology(topology, options);

                    cluster.on("connected", function () {
                        debug("Connected");
                        process.once("SIGINT", function () {
                            cluster.shutdown();
                        });
                    });

                    cluster.on("error", function (err) {
                        debug(err.stack);
                    });

                    cluster.on("exit", function () {
                        debug("Local cluster exited.");
                        process.exit();
                    });
                })();
            } else {

                // OPTION 2: Called by `storm shell`

                var _argv = _slicedToArray(argv, 3);

                var host = _argv[0];
                var port = _argv[1];
                var uploadedJarLocation = _argv[2];

                var nimbus = { host: host, port: port };

                debug("Submitting '" + options.name + "' (" + uploadedJarLocation + ") to " + host + ":" + port + ".");
                StormSubmitter.submitTopology(options.name, uploadedJarLocation, { nimbus: nimbus, config: options.config }, topology, function (err, _) {
                    if (err) {
                        throw err;
                    }
                    debug("Topology '" + options.name + "' submitted.");
                });
            }
        } else {

            // OPTION 3: Invoked as a task.
            var dispatcher = new Dispatcher(function (conf, context, done) {
                var id = context["task->component"][context.taskid];
                var component = builder.bolts[id] || builder.spouts[id] || builder.state_spouts[id];
                component.attach(this);
                component.initialize(conf, context, done);
            });

            dispatcher.run();
        }
    }

};