"use strict";

var _classCallCheck = require("babel-runtime/helpers/class-call-check")["default"];

var _inherits = require("babel-runtime/helpers/inherits")["default"];

var _get = require("babel-runtime/helpers/get")["default"];

var _createClass = require("babel-runtime/helpers/create-class")["default"];

var _core = require("babel-runtime/core-js")["default"];

var _interopRequire = require("babel-runtime/helpers/interop-require")["default"];

var Fs = _interopRequire(require("fs"));

var Path = _interopRequire(require("path"));

var EventEmitter = require("events").EventEmitter;

var SEPARATOR = "\nend\n";
var Tuple = _core.Object.freeze({

    create: function create(_ref) {
        var id = _ref.id;
        var comp = _ref.comp;
        var stream = _ref.stream;
        var task = _ref.task;
        var tuple = _ref.tuple;

        return {
            id: id,
            stream: stream,
            task: task,
            component: comp,
            values: tuple
        };
    }

});

var Dispatcher = (function (_EventEmitter) {
    function Dispatcher(initialize) {
        var _this2 = this;

        var readable = arguments[1] === undefined ? process.stdin : arguments[1];
        var writeable = arguments[2] === undefined ? process.stdout : arguments[2];

        _classCallCheck(this, Dispatcher);

        _get(_core.Object.getPrototypeOf(Dispatcher.prototype), "constructor", this).call(this);

        this.readable = readable;
        this.writeable = writeable;

        this.once("message", function (_ref) {
            var conf = _ref.conf;
            var context = _ref.context;
            var pidDir = _ref.pidDir;

            initialize.call(_this2, conf, context, function (err) {
                if (err) {
                    _this2.log(err.stack);
                    return;
                }

                _this2.on("message", function (message) {
                    if (Dispatcher.isTaskIds(message)) {
                        _this2.emit("tasks", message);
                        return;
                    }

                    if (Dispatcher.isCommand(message)) {
                        _this2.emit(message.command, message);
                        return;
                    }

                    if (Dispatcher.isHeartbeat(message)) {
                        _this2.emit("heartbeat");
                        return;
                    }

                    _this2.emit("tuple", Tuple.create(message));
                });

                var pid = Dispatcher.writePid(pidDir);
                _this2.send({ pid: pid });
            });
        });
    }

    _inherits(Dispatcher, _EventEmitter);

    _createClass(Dispatcher, {
        run: {
            value: function run() {
                var _this2 = this;

                var part = "";

                this.readable.on("readable", function () {
                    var chunk = undefined,
                        chunks = [];

                    // Read in all the data.
                    while ((chunk = _this2.readable.read()) !== null) {
                        chunks.push(chunk);
                    }

                    // Combine newly read data with any existing data (`part`)
                    chunk = Buffer.concat(chunks);
                    part += chunk.toString("utf8");

                    // Extract individual messages
                    var messages = part.split(SEPARATOR);

                    // If there are no terminators (e.g. length === 1 or 0),
                    // assume there are no complete messages, thus there
                    // is no work to do.
                    if (messages.length > 1) {
                        // Last message is either empty string (part ended
                        // with separator), or non-empty string (partial
                        // message awaiting remaining content and separator).
                        // Either way, keep that for next read operation.
                        part = messages.pop();

                        // The remaining contents of the array are messages,
                        // so emit those.
                        var _iteratorNormalCompletion = true;
                        var _didIteratorError = false;
                        var _iteratorError = undefined;

                        try {
                            for (var _iterator = _core.$for.getIterator(messages), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                                var message = _step.value;

                                _this2.emit("message", JSON.parse(message));
                            }
                        } catch (err) {
                            _didIteratorError = true;
                            _iteratorError = err;
                        } finally {
                            try {
                                if (!_iteratorNormalCompletion && _iterator["return"]) {
                                    _iterator["return"]();
                                }
                            } finally {
                                if (_didIteratorError) {
                                    throw _iteratorError;
                                }
                            }
                        }
                    }
                });
            }
        },
        send: {
            value: function send(message) {
                this.writeable.write(JSON.stringify(message) + SEPARATOR, "utf8");
            }
        }
    }, {
        isHeartbeat: {
            value: function isHeartbeat(_ref) {
                var task = _ref.task;
                var stream = _ref.stream;

                return task === -1 && stream === "__heartbeat";
            }
        },
        isTaskIds: {
            value: function isTaskIds(message) {
                return Array.isArray(message);
            }
        },
        isCommand: {
            value: function isCommand(message) {
                return "command" in message;
            }
        },
        writePid: {
            value: function writePid(dir) {
                var pid = process.pid;
                var pidfile = Path.join(dir, String(pid));

                Fs.closeSync(Fs.openSync(pidfile, "w"));

                return pid;
            }
        }
    });

    return Dispatcher;
})(EventEmitter);

module.exports = Dispatcher;