"use strict";

var _classCallCheck = require("babel-runtime/helpers/class-call-check")["default"];

var _inherits = require("babel-runtime/helpers/inherits")["default"];

var _get = require("babel-runtime/helpers/get")["default"];

var _createClass = require("babel-runtime/helpers/create-class")["default"];

var _slicedToArray = require("babel-runtime/helpers/sliced-to-array")["default"];

var _core = require("babel-runtime/core-js")["default"];

var _interopRequire = require("babel-runtime/helpers/interop-require")["default"];

var Os = _interopRequire(require("os"));

var Fs = _interopRequire(require("fs"));

var Net = _interopRequire(require("net"));

var Path = _interopRequire(require("path"));

var debuglog = _interopRequire(require("debuglog"));

var Child = _interopRequire(require("child_process"));

var EventEmitter = require("events").EventEmitter;

var Utils = _interopRequire(require("./utils"));

var StormSubmitter = _interopRequire(require("./storm_submitter"));

var debug = debuglog("cyclone");

var LocalCluster = (function (_EventEmitter) {
    function LocalCluster() {
        _classCallCheck(this, LocalCluster);

        _get(_core.Object.getPrototypeOf(LocalCluster.prototype), "constructor", this).call(this);
        this.running = _core.Promise.resolve([]);
    }

    _inherits(LocalCluster, _EventEmitter);

    _createClass(LocalCluster, {
        submitTopology: {
            value: function submitTopology(topology) {
                var _this = this;

                var options = arguments[1] === undefined ? {} : arguments[1];
                var _options$nimbus = options.nimbus;
                var nimbus = _options$nimbus === undefined ? {} : _options$nimbus;
                var _nimbus$host = nimbus.host;
                var host = _nimbus$host === undefined ? "127.0.0.1" : _nimbus$host;
                var _nimbus$port = nimbus.port;
                var port = _nimbus$port === undefined ? 6627 : _nimbus$port;

                this.running = this.startService("dev-zookeeper", 2181).then(function (proc) {
                    debug("Zookeeper running. Starting remaining services.");
                    return _core.Promise.all([proc, _this.startService("nimbus", port), _this.startService("drpc", 3772), _this.startService("supervisor"), _this.startService("ui")]);
                }).then(function (services) {
                    _this.emit("connected");

                    var tmp = Path.join(Os.tmpdir(), "cyclone-" + Utils.randInt(0, 10000000));
                    var tmpres = Path.join(tmp, "resources");

                    debug("mkdir " + tmpres);
                    Utils.mkdirp(tmpres);

                    debug("cp " + tmpres);
                    Utils.cp(".", tmpres);

                    //debug('storm shell -c nimbus.host=127.0.0.1 resources/ node resources/');
                    var entrypoint = Path.join("resources", Utils.relative(require.main.filename));

                    var flags = ["-c", "nimbus.host=" + host, "-c", "nimbus.port=" + port];
                    var args = ["shell", null, "resources/", "node", entrypoint];
                    args.splice.apply(args, [1, 1].concat(flags));

                    var storm = Child.spawn("storm", args, { stdio: ["ignore"], cwd: tmp });

                    var _storm$stdio = _slicedToArray(storm.stdio, 3);

                    var _ = _storm$stdio[0];
                    var stdout = _storm$stdio[1];
                    var stderr = _storm$stdio[2];

                    stdout.pipe(process.stdout);
                    stderr.pipe(process.stderr);

                    services.push({ name: "storm", proc: storm });
                    return services;
                })["catch"](function (err) {
                    _this.running = _core.Promise.resolve([]);
                    _this.emit("error", err);
                });
            }
        },
        shutdown: {
            value: function shutdown() {
                var _this = this;

                this.running.then(function (services) {
                    var kill = services.map(function (_ref) {
                        var name = _ref.name;
                        var proc = _ref.proc;

                        return new _core.Promise(function (resolve) {

                            debug("Shutting down " + name + ".");

                            proc.kill("SIGINT");
                            proc.once("exit", function (signal) {
                                debug("" + name + " exited with code " + signal);
                                resolve();
                            });
                        });
                    });

                    _core.Promise.all(kill).then(function () {
                        return _this.emit("exit");
                    })["catch"](function (err) {
                        return _this.emit("error", err);
                    });
                });
            }
        },
        startService: {
            value: function startService(name, port) {
                debug("Starting " + name + ".");

                var args = [name, "-c", "dev.zookeeper.path=storm-local/zookeeper", "-c", "storm.local.dir=storm-local"];
                var proc = Child.spawn("storm", args, { stdio: ["ignore"], cwd: Os.tmpdir() });

                var _proc$stdio = _slicedToArray(proc.stdio, 3);

                var _ = _proc$stdio[0];
                var stdout = _proc$stdio[1];
                var stderr = _proc$stdio[2];

                stdout.pipe(process.stdout);
                stderr.pipe(process.stderr);

                var resolved = _core.Promise.resolve({ name: name, proc: proc });
                return !isNaN(Number(port)) ? this.onReady(name, port).then(function () {
                    return resolved;
                }) : resolved;
            }
        },
        onReady: {
            value: function onReady(name, port) {
                var timeout = 60000;
                var start = Date.now();
                var deferred = Utils.defer();

                (function connect() {
                    setTimeout(function () {

                        var conn = Net.connect({ port: port });

                        conn.once("error", function (err) {
                            if (Date.now() - start > timeout) {
                                debug("Unable to connect to " + name + ". Exiting.");
                                deferred.reject(err);
                                return;
                            }
                            connect();
                        });

                        conn.once("connect", function () {
                            debug("Connected successfully to " + name + ":" + port + ".");
                            conn.end();
                        });

                        conn.once("close", function (had_error) {
                            if (!had_error) {
                                deferred.resolve();
                            }
                        });
                    }, 1000);
                })();

                return deferred.promise;
            }
        }
    });

    return LocalCluster;
})(EventEmitter);

module.exports = LocalCluster;