"use strict";

var _classCallCheck = require("babel-runtime/helpers/class-call-check")["default"];

var _createClass = require("babel-runtime/helpers/create-class")["default"];

var _core = require("babel-runtime/core-js")["default"];

var _interopRequire = require("babel-runtime/helpers/interop-require")["default"];

var Path = _interopRequire(require("path"));

var Utils = _interopRequire(require("./utils"));

var Storm = (function () {
    function Storm() {
        var _this = this;

        _classCallCheck(this, Storm);

        this.dispatcher = null;
        this.config = null;
        this.context = null;

        this.taskIdsCallbacks = [];
        this.ontasks = function (taskIds) {
            var callback = _this.taskIdsCallbacks.shift();
            callback(taskIds);
        };
    }

    _createClass(Storm, {
        execution_command: {
            get: function () {
                return "node";
            }
        },
        script: {
            get: function () {
                // The script that invokes the topology (either in local,
                // submit, or run mode) is always the entry point. Here
                // we take that module main file and get a path to it
                // relative to the module root. The expectation here is
                // that a given module's declared "main" is the process
                // entry point for a given topology, regardless of whether
                // or not the topology and associated `run` is contained
                // within that file.
                return Utils.relative(require.main.filename);
            }
        },
        initialize: {
            value: function initialize(config, context, done) {
                this.config = config;
                this.context = context;
                this.prepare(config, context, done);
            }
        },
        prepare: {

            /* Task API */

            value: function prepare(config, context, done) {
                done();
            }
        },
        declareOutputFields: {
            /* End Task API */

            /* Component API*/

            value: function declareOutputFields() {
                return [];
            }
        },
        getComponentConfiguration: {
            value: function getComponentConfiguration() {
                return {};
            }
        },
        attach: {
            /* End Component API */

            value: function attach(dispatcher) {
                this.dispatcher = dispatcher;
                this.dispatcher.on("tasks", this.ontasks);
            }
        },
        detach: {
            value: function detach() {
                this.dispatcher.removeEventListener("tasks", this.ontasks);
                this.dispatcher = null;
            }
        },
        send: {
            value: function send(message) {
                this.dispatcher.send(message);
            }
        },
        sync: {
            value: function sync() {
                this.send({ command: "sync" });
            }
        },
        debug: {
            value: function debug(msg) {
                if (this.config["topology.debug"]) {
                    this.log(msg);
                }
            }
        },
        log: {
            value: function log(msg) {
                if (typeof msg !== "string") {
                    try {
                        msg = JSON.stringify(msg);
                    } catch (err) {
                        msg = String(msg);
                    }
                }
                this.send({ command: "log", msg: msg });
            }
        },
        emit: {
            value: function emit(message) {
                var callback = arguments[1] === undefined ? Function.prototype : arguments[1];

                if (message.task) {

                    // Just fulfill with the provided task
                    setImmediate(function () {
                        callback([message.task]);
                    });
                } else {

                    // Await response from Storm
                    this.taskIdsCallbacks.push(callback);
                }

                var transformed = this.transformMessage(message);
                var payload = _core.Object.assign({ command: "emit" }, transformed);
                this.send(payload);
            }
        },
        transformMessage: {
            value: function transformMessage(message) {
                return message;
            }
        }
    });

    return Storm;
})();

module.exports = Storm;