"use strict";

var _classCallCheck = require("babel-runtime/helpers/class-call-check")["default"];

var _createClass = require("babel-runtime/helpers/create-class")["default"];

var _slicedToArray = require("babel-runtime/helpers/sliced-to-array")["default"];

var _core = require("babel-runtime/core-js")["default"];

var _interopRequire = require("babel-runtime/helpers/interop-require")["default"];

var Iterator = _interopRequire(require("./iterator"));

var _thrift = require("thrift");

var TFramedTransport = _thrift.TFramedTransport;
var TBufferedTransport = _thrift.TBufferedTransport;
var TBinaryProtocol = _thrift.TBinaryProtocol;

var _generatedStorm_types = require("./generated/storm_types");

var Bolt = _generatedStorm_types.Bolt;
var SpoutSpec = _generatedStorm_types.SpoutSpec;
var ShellComponent = _generatedStorm_types.ShellComponent;
var ComponentCommon = _generatedStorm_types.ComponentCommon;
var ComponentObject = _generatedStorm_types.ComponentObject;
var Grouping = _generatedStorm_types.Grouping;
var StreamInfo = _generatedStorm_types.StreamInfo;
var StormTopology = _generatedStorm_types.StormTopology;
var GlobalStreamId = _generatedStorm_types.GlobalStreamId;
var NullStruct = _generatedStorm_types.NullStruct;

var TopologyBuilder = (function () {
    function TopologyBuilder() {
        _classCallCheck(this, TopologyBuilder);

        this.bolts = Object.create(null);
        this.spouts = Object.create(null);
        this.state_spouts = Object.create(null);
        this.common = Object.create(null);
    }

    _createClass(TopologyBuilder, {
        setBolt: {
            value: function setBolt(id, bolt, parallelismHint) {
                this.validateUnusedId(id);
                this.initCommon(id, bolt, parallelismHint);
                this.bolts[id] = bolt;

                return new BoltGetter(this, id);
            }
        },
        setSpout: {
            value: function setSpout(id, spout, parallelismHint) {
                this.validateUnusedId(id);
                this.initCommon(id, spout, parallelismHint);
                this.spouts[id] = spout;
            }
        },
        setStateSpout: {
            value: function setStateSpout() {
                throw new Error("Not implemented.");
            }
        },
        createTopology: {

            //write(callback) {
            //    function complete(buffer) {
            //        callback(null, buffer);
            //    }
            //
            //    let transport = new TFramedTransport(null, complete);
            //    let protocol = new TBinaryProtocol(transport);
            //    let topology = this.createTopology();
            //    topology.write(protocol);
            //    protocol.flush();
            //}
            //
            //read(bytes, callback) {
            //    TFramedTransport.receiver(function (transport) {
            //        var protocol = new TBinaryProtocol(transport);
            //        let topology = new StormTopology();
            //        topology.read(protocol);
            //        callback(null, topology);
            //    })(bytes);
            //}

            value: function createTopology() {
                var bolts = {};
                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                    for (var _iterator = _core.$for.getIterator(Iterator.entries(this.bolts)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var _step$value = _slicedToArray(_step.value, 2);

                        var id = _step$value[0];
                        var bolt = _step$value[1];

                        var shell = new ShellComponent();
                        shell.execution_command = bolt.execution_command;
                        shell.script = bolt.script;

                        var component = new ComponentObject();
                        component.shell = shell;

                        var b = new Bolt();
                        b.bolt_object = component;
                        b.common = this.getComponentCommon(id, bolt);

                        bolts[id] = b;
                    }
                } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion && _iterator["return"]) {
                            _iterator["return"]();
                        }
                    } finally {
                        if (_didIteratorError) {
                            throw _iteratorError;
                        }
                    }
                }

                var spouts = {};
                var _iteratorNormalCompletion2 = true;
                var _didIteratorError2 = false;
                var _iteratorError2 = undefined;

                try {
                    for (var _iterator2 = _core.$for.getIterator(Iterator.entries(this.spouts)), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                        var _step2$value = _slicedToArray(_step2.value, 2);

                        var id = _step2$value[0];
                        var spout = _step2$value[1];

                        var shell = new ShellComponent();
                        shell.execution_command = spout.execution_command;
                        shell.script = spout.script;

                        var component = new ComponentObject();
                        component.shell = shell;

                        var s = new SpoutSpec();
                        s.spout_object = component;
                        s.common = this.getComponentCommon(id, spout);

                        spouts[id] = s;
                    }
                } catch (err) {
                    _didIteratorError2 = true;
                    _iteratorError2 = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion2 && _iterator2["return"]) {
                            _iterator2["return"]();
                        }
                    } finally {
                        if (_didIteratorError2) {
                            throw _iteratorError2;
                        }
                    }
                }

                return new StormTopology({ bolts: bolts, spouts: spouts, state_spouts: {} });
            }
        },
        validateUnusedId: {
            value: function validateUnusedId(id) {
                if (id in this.bolts) {
                    throw new Error("Bolt has already been declared for id " + id);
                }

                if (id in this.spouts) {
                    throw new Error("Spout has already been declared for id " + id);
                }

                if (id in this.state_spouts) {
                    throw new Error("State spout has already been declared for id " + id);
                }
            }
        },
        getComponentCommon: {
            value: function getComponentCommon(id, component) {
                var stream = new StreamInfo();
                stream.output_fields = component.declareOutputFields() || [];
                stream.direct = false;

                var common = this.common[id];
                common.streams["default"] = stream;
                return common;
            }
        },
        initCommon: {
            value: function initCommon(id, component, parallelismHint) {
                var common = new ComponentCommon();

                common.inputs = new _core.Map();
                common.streams = {};

                if (!isNaN(Number(parallelismHint))) {
                    common.parallelism_hint = parallelismHint;
                }

                var conf = component.getComponentConfiguration() || {};
                common.json_conf = JSON.stringify(conf);

                this.common[id] = common;
            }
        }
    });

    return TopologyBuilder;
})();

module.exports = TopologyBuilder;

var BoltGetter = (function () {
    function BoltGetter(parent, boltId) {
        _classCallCheck(this, BoltGetter);

        this.parent = parent;
        this.boltId = boltId;
    }

    _createClass(BoltGetter, {
        fieldsGrouping: {
            value: function fieldsGrouping(componentId, streamId) {
                var fields = arguments[2] === undefined ? [] : arguments[2];

                if (Array.isArray(streamId)) {
                    fields = streamId;
                    streamId = "default";
                }

                return this.grouping(componentId, streamId, "fields", fields);
            }
        },
        globalGrouping: {
            value: function globalGrouping(componentId) {
                var streamId = arguments[1] === undefined ? "default" : arguments[1];

                return this.fieldsGrouping(componentId, streamId);
            }
        },
        shuffleGrouping: {
            value: function shuffleGrouping(componentId) {
                var streamId = arguments[1] === undefined ? "default" : arguments[1];

                return this.grouping(componentId, streamId, "shuffle", new NullStruct());
            }
        },
        localOrShuffleGrouping: {
            value: function localOrShuffleGrouping(componentId) {
                var streamId = arguments[1] === undefined ? "default" : arguments[1];

                return this.grouping(componentId, streamId, "local_or_shuffle", new NullStruct());
            }
        },
        noneGrouping: {
            value: function noneGrouping(componentId) {
                var streamId = arguments[1] === undefined ? "default" : arguments[1];

                return this.grouping(componentId, streamId, "none", new NullStruct());
            }
        },
        allGrouping: {
            value: function allGrouping(componentId) {
                var streamId = arguments[1] === undefined ? "default" : arguments[1];

                return this.grouping(componentId, streamId, "all", new NullStruct());
            }
        },
        directGrouping: {
            value: function directGrouping(componentId) {
                var streamId = arguments[1] === undefined ? "default" : arguments[1];

                return this.grouping(componentId, streamId, "direct", new NullStruct());
            }
        },
        grouping: {
            value: (function (_grouping) {
                var _groupingWrapper = function grouping(_x, _x2, _x3, _x4) {
                    return _grouping.apply(this, arguments);
                };

                _groupingWrapper.toString = function () {
                    return _grouping.toString();
                };

                return _groupingWrapper;
            })(function (componentId, streamId, attribute, grouping) {
                var globalStreamId = new GlobalStreamId({ componentId: componentId, streamId: streamId });

                var g = {};
                g[attribute] = grouping;

                this.parent.common[this.boltId].inputs.set(globalStreamId, new Grouping(g));
                return this;
            })
        },
        customGrouping: {
            value: function customGrouping() {
                throw new Error("Not implemented");
            }
        }
    });

    return BoltGetter;
})();