"use strict";

var _core = require("babel-runtime/core-js")["default"];

var _interopRequire = require("babel-runtime/helpers/interop-require")["default"];

var Fs = _interopRequire(require("fs"));

var Path = _interopRequire(require("path"));

var Https = _interopRequire(require("https"));

var Espree = _interopRequire(require("espree"));

var Child = _interopRequire(require("child_process"));

var _fsExtra = require("fs-extra");

var mkdirp = _fsExtra.mkdirpSync;
var rimraf = _fsExtra.removeSync;

// TODO: Detect version of storm installed to determine appropriate thrift file.
var URL = "https://raw.githubusercontent.com/apache/storm/v0.9.3/storm-core/src/storm.thrift";
var THRIFT = "scripts/storm.thrift";
var DEST = "lib/generated";
var PATCH = "scripts/storm_types.patch";

// Clean up previous runs and create dest dir
console.log("rm " + THRIFT);
rimraf(THRIFT);

console.log("rm " + DEST);
rimraf(DEST);
mkdirp(DEST);

console.log("get " + URL);
Https.get(URL, function (res) {
    res.pipe(Fs.createWriteStream(THRIFT)).on("finish", function () {
        return generate(THRIFT, DEST);
    });
});

// Use thrift to generate JS from the provided thrift file.
function generate(thriftFile, dest) {
    console.log("gen " + thriftFile);

    var child = undefined;
    child = Child.spawn("thrift", ["-gen", "js:node", "-out", dest, thriftFile]);
    child.on("exit", function () {
        return rewrite(dest);
    });
}

// Apply `exports` to each file in the directory and then apple the
// provided patch.
function rewrite(dir) {
    eachfile(dir, exports);

    // Applying Map patch
    console.log("git apply " + PATCH);
    Child.spawn("git", ["apply", PATCH], { stdio: ["pipe", process.stdout, process.stderr] });
}

// Simple helper to recursively traverse a directory
// and invoke a fn with each located file.
function eachfile(dir, fn) {

    var files = Fs.readdirSync(dir);

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = _core.$for.getIterator(files), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var file = _step.value;

            var path = Path.join(dir, file);
            var stat = Fs.statSync(path);

            if (stat.isDirectory()) {
                eachfile(path, fn);
            } else {
                fn(path);
            }
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator["return"]) {
                _iterator["return"]();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }
}

// Converts the generated thrift JS into valid 'strict'-safe
// JS by adding variable declarations for all top-level assignment
// expressions to the beginning of each file.
function exports(file) {
    var js = Fs.readFileSync(file, "utf8");
    var ast = Espree.parse(js);
    var identifiers = [];

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = _core.$for.getIterator(ast.body), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var _step$value = _step.value;
            var type = _step$value.type;
            var _step$value$expression = _step$value.expression;
            var expression = _step$value$expression === undefined ? {} : _step$value$expression;

            if (type === "ExpressionStatement") {
                var expressionType = expression.type;
                var left = expression.left;

                if (expressionType === "AssignmentExpression" && left.type === "Identifier") {
                    identifiers.push(left.name);
                }
            }
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator["return"]) {
                _iterator["return"]();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    console.log("prepend " + file);
    var declarations = "var " + identifiers.join(",\n    ") + ";\n";
    Fs.writeFileSync(file, declarations + js);
}